#ifndef TFIELDSELECTION
#define TFIELDSELECTION

// ROOT headers
#include <Rtypes.h>
// STL headers
#include <string>
#include <vector>
#include <iostream>

/** \class TFieldSelection
* \brief A helper class for TTextToData which manages selection of data fields from a text file.
*
* This class is parses an input string containing non-negative integers, commas and hyphens e.g. "0,2,4-7".
* Hyphens indicate field ranges, so the example string would select fields 0, 2, and all fields between 4 and 
* 7 inclusive.
*
* \author Hugh Dickinson
*/
class TFieldSelection {

public :
/// Default Constructor
TFieldSelection(){}
/// Constructor taking a string containing a comma and hyphen separated list of fields.
TFieldSelection(std::string fields);
/// Constructor taking a character array containing a comma and hyphen separated list of fields.
TFieldSelection(const char * fields);
/// Destructor
virtual ~TFieldSelection(){}
/// Should the passed field be extracted?
virtual bool IsRequiredField(int field);
/// Get the vector of required fields.
const std::vector<int>& GetFields(){return fFields;}

private :
/// Process the field string and insert the values into the fFields set.
void ParseFields(std::string fields);
/// Internal vector of required fields.
std::vector<Int_t> fFields;

ClassDef(TFieldSelection, 1)

};

/** \class TSelectAll
* \brief A subclass of TFieldSelection which selects all fields within a text file.
*
* This class is overrides the IsRequiredField() method of TFieldSelection such that all 
* fields are selected.
*
* \author Hugh Dickinson
*/
class TSelectAll : public TFieldSelection {
public :
/// Default Constructor
TSelectAll():TFieldSelection(){}
/// Should the passed field be extracted?
virtual bool IsRequiredField(int field){return true;}
/// Destructor
virtual ~TSelectAll(){}

ClassDef(TSelectAll, 1)
};

/** \class TFieldSelector
* \brief A factory class which produces a TFieldSelection or a TSelectAll depending
* upon a string input.
*
* This factory class implements the GetSelector method which returns a TFieldSelection 
* or a TSelectAll depending upon the classes construction input. The default is to select 
* all fields.
*
* \author Hugh Dickinson
*/
class TFieldSelector {

public :

/// Default constructor
TFieldSelector(const char * fields = "allFields");
/// Destructor
virtual ~TFieldSelector(){}
/// Return the appropriate selection object.
TFieldSelection * GetSelection();

private :
/// The input field string.
std::string fFields;

ClassDef(TFieldSelector,0);

};

#endif // TFIELDSELECTION
