#ifndef TTEXTTODATA
#define TTEXTTODATA

#include "TFieldSelection.h"
// ROOT headers
#include <Rtypes.h>
// STL headers
#include <fstream>
#include <iterator>
#include <stdexcept>
#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <map>

/** \class TTextToData
* \brief Class template which parses a text table and returns selected rows or columns on demand.
*
* This class is primarily designed to simplfify the parsing of text files containing tables of numerical 
* data, although since it is a template, and form of whitespace delimited data could be parsed in principle.
*
* \todo Optional row-major or column-major storage.
*
* \author Hugh Dickinson
*/
template <typename NumericType>
class TTextToData {
	
public :
	
	/// Default Constructor.
	TTextToData(){}
	/// Constructor taking the filename, as well as column and row selections as pointers.
	TTextToData(std::string fileName, TFieldSelector& cols, TFieldSelector& rows);
	/// Destructor.
	virtual ~TTextToData();
	
	/// Retrieve a specified row.
	std::vector<NumericType> * GetRow(int row);
	/// Retrieve a specified column.
	std::vector<NumericType> * GetCol(int col);
	
	/// Print a specified row.
	void PrintRow(int row);
	/// Print a specified column.
	void PrintCol(int col);
	/// Print available rows.
	void PrintStoredRows();
	/// Print available columns.
	void PrintStoredCols();
	
private :
		
	/// Parse the input file and extract the required data.
	void ParseFile();
	
	/// The input file name.
	std::string fFileName;
	/// The current row selection.
	TFieldSelection * fRows;
	/// The current column selection.
	TFieldSelection * fCols;
	/// Internal data storage structure.
	std::map<Int_t, std::map<Int_t, NumericType> > fStorage;
	
	ClassDef(TTextToData, 0);
	
};

template <typename NumericType>
TTextToData<NumericType>::TTextToData(std::string fileName, TFieldSelector& cols, TFieldSelector& rows):
fFileName(fileName),
fRows(rows.GetSelection()),
fCols(cols.GetSelection())
{
	try{
		ParseFile();
	}
	catch(std::runtime_error& e){
		std::cerr << "TTextToData error: " << e.what() << "." << std::endl;
		throw std::runtime_error("Runtime error in TFieldSelection");
	}
}

template <typename NumericType>
TTextToData<NumericType>::~TTextToData(){
	delete fCols, fRows;
}

template <typename NumericType>
void TTextToData<NumericType>::ParseFile(){
	std::cout << "Processing " << fFileName << "..." << std::flush;
	std::ifstream fileStream(fFileName.c_str());
	if(fileStream.is_open()){
		// parse the file
		std::string fileLine;
		std::stringstream fileLineStream;
		NumericType data;
		int iRow(0);
		while(!fileStream.eof()){
			std::getline(fileStream, fileLine);
			if(fRows->IsRequiredField(iRow)){
				// extract the row
				fileLineStream.str("");
				fileLineStream.clear();
				fileLineStream << fileLine;
				int iCol(0);
				while(!fileLineStream.eof()){
					fileLineStream >> data;
					if(fileLineStream.fail()){
						throw std::runtime_error("Extracted data cannot be converted to the correct type");
					}
					if(fCols->IsRequiredField(iCol)){
						// extract the field
						fStorage[iRow][iCol] = data; 
					}
					iCol++;
				}
			}
			iRow++;
		}
	}
	else{
		std::cout << std::endl;
		throw std::runtime_error("Could not open the specified file");
	}
	std::cout << "done." << std::endl;
}

template <typename NumericType>
std::vector<NumericType> * TTextToData<NumericType>::GetRow(int row){
	if(fStorage.find(row) != fStorage.end()){
		// requested row exists
		std::vector<NumericType> * output = new std::vector<NumericType>;
		typename std::map<int, NumericType>::iterator fieldIt;
		for(fieldIt = (fStorage[row]).begin(); fieldIt != (fStorage[row]).end(); ++fieldIt){
			output->push_back(fieldIt->second);
		}
		return output;
	}
	return 0;
}

template <typename NumericType>
std::vector<NumericType> * TTextToData<NumericType>::GetCol(int col){
	if(fStorage.begin()->second.find(col) != fStorage.begin()->second.end()){
		// requested column exists
		std::vector<NumericType> * output = new std::vector<NumericType>;
		typename std::map<int, std::map<int, NumericType> >::iterator rowIt;
		for(rowIt = fStorage.begin(); rowIt != fStorage.end(); ++rowIt){
			output->push_back((rowIt->second)[col]);
		} 
		return output;
	}
	return 0;	
}

template <typename NumericType>
void TTextToData<NumericType>::PrintRow(int row){
	std::vector<NumericType> * rowData = GetRow(row);
	if(rowData){
		std::cout << "Row " << row << ":" << std::endl;
		std::copy(rowData->begin(), rowData->end(), std::ostream_iterator<NumericType>(std::cout, "\n"));
	}
	else{
		std::cout << "Row " << row << " has not been extracted." << std::endl;
	}
	delete rowData;
}

template <typename NumericType>
void TTextToData<NumericType>::PrintCol(int col){
	std::vector<NumericType> * colData = GetCol(col);
	if(colData){
		std::cout << "Column " << col << ":" << std::endl;
		std::copy(colData->begin(), colData->end(), std::ostream_iterator<NumericType>(std::cout, "\n"));
	}
	else{
		std::cout << "Column " << col << " has not been extracted." << std::endl;
	}
	delete colData;
}

template <typename NumericType>
void TTextToData<NumericType>::PrintStoredRows(){
	std::cout << "Available Rows:" << std::endl;
	typename std::map<int, std::map<int, NumericType> >::iterator rowIt;
	for(rowIt = fStorage.begin(); rowIt != fStorage.end(); ++rowIt){
		std::cout << rowIt->first << std::endl;
	}
}

template <typename NumericType>
void TTextToData<NumericType>::PrintStoredCols(){
	std::cout << "Available Columns:" << std::endl;
	if(!fStorage.empty()){
		std::map<int, NumericType> firstRow = fStorage.begin()->second;
		typename std::map<int, NumericType>::iterator colIt;
		for(colIt = firstRow.begin(); colIt != firstRow.end(); ++colIt){
			std::cout << colIt->first << std::endl;
		}
	}
}

// Useful typedefs for common data types.
typedef TTextToData<Double_t> TTextToDoubles;
typedef TTextToData<Int_t> TTextToInts;


#endif TTEXTTODATA
